//
//  OPHBluetoothService.h
//
//  Copyright (c) 2022年 OPTOELECTRONICS CO., LTD. All rights reserved.
//


#import <Foundation/Foundation.h>
#import <ExternalAccessory/ExternalAccessory.h>

@class OPHBluetoothService;

//**************************************************************************************************
// @name OPHSessionControllerDelegate <NSObject>
//**************************************************************************************************


/** OPHSessionControllerのデリゲートです。
 */
@protocol OPHBluetoothServiceDelegate <NSObject>
@optional
/** 結果を受信したことを通知します。
 *  @param service OPHBluetoothServiceのインスタンスです。
 *  @param data 受信した結果です
 */- (void)bluetoothService:(OPHBluetoothService *)service receivedData:(NSData *)data;
@end


//**************************************************************************************************
// @name OPHSessionController
//**************************************************************************************************


/** OPH-5000iとのI/O処理、セッション管理、アクセサリ保持を担当するコアクラスです。
 * このクラスはアプリに1つのみ存在するSingletonクラスです。
 */
@interface OPHBluetoothService : NSObject <EAAccessoryDelegate, NSStreamDelegate>

/** OPHBluetoothServiceのインスタンスを返します。
 * @return OPHBluetoothServiceのインスタンスを返します。
 */
+ (OPHBluetoothService *)sharedController;
/** 指定したEAAccesoryとprotocolStringを初期化します。
 * @param accessory アクセサリを指定します。
 * @param protocolString OPH-5000iのプロトコルストリングを指定します。
 */
- (void)setupControllerForAccessory:(EAAccessory *)accessory withProtocolString:(NSString *)protocolString;

/** EASessionを生成し、NSInputStreamとNSOutputStreamをオープンします。
 *
 * 1. 現在保持しているEAAccessoryに自分自身をデリゲートとして設定します。
 * 2. EASessionを生成し取得します。生成時には、setupControllerForAccessory:accessory:withProtocolString
 *    で指定したEAAccessoryとprotocolStringを使います。
 *
 * @return BOOL EASessionを生成できた場合、YESを返します。それ以外はNOを返します。
 */
- (BOOL)openSession;

/** EASessionをクローズし、RunLoopを削除します。EASessionのリリースし、nilで初期化します。
 */
- (void)closeSession;

/**  OPH-5000iにデータを送信します。
 * @param data  送信するデータ
 * アクセサリインターフェース仕様によりOPH-5000i(MFiアクセサリ)がiOSデバイスから受信可能なExternal Accessory Protocol メッセージの最大長は2036 bytesです。
 * @return 送信成功したらYESをを返します。data.lengthが2036bytes以上を送信するとNOを返します。
 */
- (BOOL)writeData:(NSData *)data;

/** 接続しているEAAccessory
 */
@property (weak, nonatomic, readonly) EAAccessory *accessory;

/** 接続しているデバイスのprotocolString
 */
@property (nonatomic, readonly) NSString *protocolString;

/** データ受信後に実行されるデリゲート
 */
@property (nonatomic, weak) id<OPHBluetoothServiceDelegate> delegate;


@end
